@@ -58,6 +58,9 @@ public class EditorGraph extends JGraph {
 	}
 
 	public static void main(String[] args) {
+		// Switch off D3D because of Sun XOR painting bug
+		// See http://www.jgraph.com/forum/viewtopic.php?t=4066
+		System.setProperty("sun.java2d.d3d", "false");
 		JFrame frame = new JFrame("EditorGraph");
 		frame.getContentPane().add(new EditorGraph());
 		frame.pack();
