@@ -422,6 +422,19 @@ public class GraphEdX extends GraphEd {
 				parent.remove(graph);
 				XMLDecoder decoder = new XMLDecoder(new BufferedInputStream(
 						new FileInputStream(fileChooser.getSelectedFile())));
+
+				// In case you need to debug XML decoding here is how
+				// to print a stack trace
+				/*
+				decoder.setExceptionListener(new ExceptionListener()
+				{
+					public void exceptionThrown(Exception exception)
+					{
+						exception.printStackTrace();
+					}
+				});
+				*/
+				
 				graph = (JGraph) decoder.readObject();
 				// Take the marquee handler from the original graph and
 				// use it in the new graph as well.
@@ -552,6 +565,9 @@ public class GraphEdX extends GraphEd {
 	 */
 	public static void main(String[] args) {
 		try {
+			// Switch off D3D because of Sun XOR painting bug
+			// See http://www.jgraph.com/forum/viewtopic.php?t=4066
+			System.setProperty("sun.java2d.d3d", "false");
 			// Construct Frame
 			JFrame frame = new JFrame("GraphEdX");
 			// Set Close Operation to Exit
