@@ -19,6 +19,9 @@ import org.jgraph.graph.AbstractCellView;
 public class MyCellModelEditor {
 
 	public static void main(String[] args) {
+		// Switch off D3D because of Sun XOR painting bug
+		// See http://www.jgraph.com/forum/viewtopic.php?t=4066
+		System.setProperty("sun.java2d.d3d", "false");
 		JFrame frame = new JFrame("MyCellModelEditor");
 		JGraph graph = new JGraph();
 		frame.getContentPane().add(graph);
