@@ -25,6 +25,7 @@ import java.awt.Color;
 import java.awt.Cursor;
 import java.awt.Graphics;
 import java.awt.Point;
+import java.awt.Rectangle;
 import java.awt.event.ActionEvent;
 import java.awt.event.KeyEvent;
 import java.awt.event.KeyListener;
@@ -40,6 +41,7 @@ import javax.swing.Action;
 import javax.swing.BorderFactory;
 import javax.swing.ImageIcon;
 import javax.swing.JApplet;
+import javax.swing.JComponent;
 import javax.swing.JFrame;
 import javax.swing.JLabel;
 import javax.swing.JPanel;
@@ -47,6 +49,7 @@ import javax.swing.JPopupMenu;
 import javax.swing.JScrollPane;
 import javax.swing.JToolBar;
 import javax.swing.SwingUtilities;
+import javax.swing.border.BevelBorder;
 import javax.swing.event.UndoableEditEvent;
 
 import org.jgraph.JGraph;
@@ -102,6 +105,9 @@ public class GraphEd extends JApplet implements GraphSelectionListener,
 
 	// Main Method
 	public static void main(String[] args) {
+		// Switch off D3D because of Sun XOR painting bug
+		// See http://www.jgraph.com/forum/viewtopic.php?t=4066
+		System.setProperty("sun.java2d.d3d", "false");
 		// Construct Frame
 		JFrame frame = new JFrame("GraphEd");
 		// Set Close Operation to Exit
@@ -537,6 +543,30 @@ public class GraphEd extends JApplet implements GraphSelectionListener,
 		// Holds the First and the Current Port
 		protected PortView port, firstPort;
 
+		/**
+		 * Component that is used for highlighting cells if
+		 * the graph does not allow XOR painting.
+		 */
+		protected JComponent highlight = new JPanel();
+
+		public MyMarqueeHandler() {
+			// Configures the panel for highlighting ports
+			highlight = createHighlight();
+		}
+	
+		/**
+		 * Creates the component that is used for highlighting cells if
+		 * the graph does not allow XOR painting.
+		 */
+		protected JComponent createHighlight() {
+			JPanel panel = new JPanel();
+			panel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
+			panel.setVisible(false);
+			panel.setOpaque(false);
+	
+			return panel;
+		}
+		
 		// Override to Gain Control (for PopupMenu and ConnectMode)
 		public boolean isForceMarqueeEvent(MouseEvent e) {
 			if (e.isShiftDown())
@@ -624,6 +654,8 @@ public class GraphEd extends JApplet implements GraphSelectionListener,
 
 		// Connect the First Port and the Current Port in the Graph or Repaint
 		public void mouseReleased(MouseEvent e) {
+			highlight(graph, null);
+			
 			// If Valid Event, Current and First Port
 			if (e != null && port != null && firstPort != null
 					&& firstPort != port) {
@@ -658,17 +690,48 @@ public class GraphEd extends JApplet implements GraphSelectionListener,
 
 		// Use Xor-Mode on Graphics to Paint Connector
 		protected void paintConnector(Color fg, Color bg, Graphics g) {
-			// Set Foreground
-			g.setColor(fg);
-			// Set Xor-Mode Color
-			g.setXORMode(bg);
-			// Highlight the Current Port
-			paintPort(graph.getGraphics());
-			// If Valid First Port, Start and Current Point
-			if (firstPort != null && start != null && current != null)
+			if (graph.isXorEnabled()) {
+				// Set Foreground
+				g.setColor(fg);
+				// Set Xor-Mode Color
+				g.setXORMode(bg);
+				// Highlight the Current Port
+				paintPort(graph.getGraphics());
+				
+				drawConnectorLine(g);
+			} else {
+				Rectangle dirty = new Rectangle((int) start.getX(), (int) start.getY(), 1, 1);
+				
+				if (current != null) {
+					dirty.add(current);
+				}
+				
+				dirty.grow(1, 1);
+				
+				graph.repaint(dirty);
+				highlight(graph, port);
+			}
+		}
+		
+		// Overrides parent method to paint connector if
+		// XOR painting is disabled in the graph
+		public void paint(JGraph graph, Graphics g)
+		{
+			super.paint(graph, g);
+			
+			if (!graph.isXorEnabled())
+			{
+				g.setColor(Color.black);
+				drawConnectorLine(g);
+			}
+		}
+		
+		protected void drawConnectorLine(Graphics g) {
+			if (firstPort != null && start != null && current != null) {
 				// Then Draw A Line From Start to Current Point
 				g.drawLine((int) start.getX(), (int) start.getY(),
 						(int) current.getX(), (int) current.getY());
+			}
 		}
 
 		// Use the Preview Flag to Draw a Highlighted Port
@@ -690,6 +753,55 @@ public class GraphEd extends JApplet implements GraphSelectionListener,
 			}
 		}
 
+		/**
+		 * Highlights the given cell view or removes the highlight if
+		 * no cell view is specified.
+		 * 
+		 * @param graph
+		 * @param cellView
+		 */
+		protected void highlight(JGraph graph, CellView cellView)
+		{
+			if (cellView != null)
+			{
+				highlight.setBounds(getHighlightBounds(graph, cellView));
+
+				if (highlight.getParent() == null)
+				{
+					graph.add(highlight);
+					highlight.setVisible(true);
+				}
+			}
+			else
+			{
+				if (highlight.getParent() != null)
+				{
+					highlight.setVisible(false);
+					highlight.getParent().remove(highlight);
+				}
+			}
+		}
+
+		/**
+		 * Returns the bounds to be used to highlight the given cell view.
+		 * 
+		 * @param graph
+		 * @param cellView
+		 * @return
+		 */
+		protected Rectangle getHighlightBounds(JGraph graph, CellView cellView)
+		{
+			boolean offset = (GraphConstants.getOffset(cellView.getAllAttributes()) != null);
+			Rectangle2D r = (offset) ? cellView.getBounds() : cellView
+					.getParentView().getBounds();
+			r = graph.toScreen((Rectangle2D) r.clone());
+			int s = 3;
+
+			return new Rectangle((int) (r.getX() - s), (int) (r.getY() - s),
+					(int) (r.getWidth() + 2 * s), (int) (r.getHeight() + 2 * s));
+		}
+
+
 	} // End of Editor.MyMarqueeHandler
 
 	//
