@@ -1,5 +1,5 @@
 /*
- * $Id: EdgeRenderer.java,v 1.22 2008/11/13 00:14:19 david Exp $ * 
+ * $Id: EdgeRenderer.java,v 1.25 2009/04/06 19:44:25 david Exp $ * 
  * Copyright (c) 2001-2007 Gaudenz Alder
  *  
  * See LICENSE file in distribution for licensing details of this source file
@@ -19,6 +19,7 @@ import java.awt.Polygon;
 import java.awt.Rectangle;
 import java.awt.RenderingHints;
 import java.awt.Shape;
+import java.awt.geom.AffineTransform;
 import java.awt.geom.Ellipse2D;
 import java.awt.geom.GeneralPath;
 import java.awt.geom.Point2D;
@@ -44,6 +45,9 @@ import org.jgraph.util.Spline2D;
 
 public class EdgeRenderer extends JComponent implements CellViewRenderer,
 		Serializable {
+    
+    	/** Switch for exact label hit detection on rotated labels. Default is false. */
+    	public static boolean HIT_LABEL_EXACT = false;
 
 	/** Static Graphics used for Font Metrics */
 	protected static transient Graphics fontGraphics;
@@ -213,7 +217,45 @@ public class EdgeRenderer extends JComponent implements CellViewRenderer,
 			}
 			Rectangle2D r = getLabelBounds(graph, view);
 			if (r != null && r.intersects(rect))
-				return true;
+			{
+				boolean hits = true;
+
+				// Performs exact hit detection on rotated labels
+				if (HIT_LABEL_EXACT) {
+				    AffineTransform tx = g2.getTransform();
+
+				    try {
+					String lab = graph.convertValueToString(view);
+					Point2D tmpPt = getLabelPosition(view);
+					Dimension size = getLabelSize(view, lab);
+					Rectangle2D tmp = new Rectangle((int) tmpPt
+						.getX(), (int) tmpPt.getY(), size.width,
+						size.height);
+
+					double cx = tmp.getCenterX();
+					double cy = tmp.getCenterY();
+
+					g2.translate(-size.width / 2, -size.height * 0.75
+						- metrics.getDescent());
+
+					boolean applyTransform = isLabelTransform(lab);
+					double angle = 0;
+
+					if (applyTransform) {
+					    angle = getLabelAngle(lab);
+					    g2.rotate(angle, cx, cy);
+					}
+
+					hits = g2.hit(rect, tmp, false);
+				    } finally {
+					g2.setTransform(tx);
+				    }
+				}
+
+				if (hits) {
+				    return true;
+				}
+			}
 			Object[] labels = GraphConstants.getExtraLabels(view
 					.getAllAttributes());
 			if (labels != null) {
@@ -569,72 +611,125 @@ public class EdgeRenderer extends JComponent implements CellViewRenderer,
 				Graphics2D g2 = (Graphics2D) g;
 				g2.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL,
 						RenderingHints.VALUE_STROKE_PURE);
-				int c = BasicStroke.CAP_BUTT;
-				int j = BasicStroke.JOIN_MITER;
 				setOpaque(false);
 				super.paint(g);
 				translateGraphics(g);
-				g.setColor(getForeground());
-				if (lineWidth > 0) {
-					g2.setStroke(new BasicStroke(lineWidth, c, j));
-					if (gradientColor != null && !preview) {
-						g2.setPaint(new GradientPaint(0, 0, getBackground(),
-								getWidth(), getHeight(), gradientColor, true));
-					}
-					if (view.beginShape != null) {
-						if (beginFill)
-							g2.fill(view.beginShape);
-						g2.draw(view.beginShape);
-					}
-					if (view.endShape != null) {
-						if (endFill)
-							g2.fill(view.endShape);
-						g2.draw(view.endShape);
-					}
-					if (lineDash != null) // Dash For Line Only
-						g2.setStroke(new BasicStroke(lineWidth, c, j, 10.0f,
-								lineDash, dashOffset));
-					if (view.lineShape != null)
-						g2.draw(view.lineShape);
-				}
-
-				if (selected) { // Paint Selected
-					g2.setStroke(GraphConstants.SELECTION_STROKE);
-					g2.setColor(highlightColor);
-					if (view.beginShape != null)
-						g2.draw(view.beginShape);
-					if (view.lineShape != null)
-						g2.draw(view.lineShape);
-					if (view.endShape != null)
-						g2.draw(view.endShape);
-				}
-				g2.setStroke(new BasicStroke(1));
-				g
-						.setFont((extraLabelFont != null) ? extraLabelFont
-								: getFont());
-				Object[] labels = GraphConstants.getExtraLabels(view
-						.getAllAttributes());
-				JGraph graph = (JGraph)this.graph.get();
-				if (labels != null) {
-					for (int i = 0; i < labels.length; i++)
-						paintLabel(g, graph.convertValueToString(labels[i]),
-								getExtraLabelPosition(view, i),
-								false || !simpleExtraLabels);
-				}
-				if (graph.getEditingCell() != view.getCell()) {
-					g.setFont(getFont());
-					Object label = graph.convertValueToString(view);
-					if (label != null) {
-						paintLabel(g, label.toString(), getLabelPosition(view),
-								true);
-					}
-				}
+				// Hook for pre-painting
+				beforeEdgePaint(g);
+				// Actual edge drawing
+				paintEdge(g);
+				// Drawing of any selection
+				paintSelection(g);
+				// Drawing of labels
+				paintLabels(g);
+				// Hook for post-painting
+				afterEdgePaint(g);
 			}
 		} else {
 			paintSelectionBorder(g);
 		}
 	}
 
+	/**
+	 * Draws the edge labels
+	 * @param g the graphics object being painted to
+	 */
+	protected void paintLabels(Graphics g)
+	{
+		Graphics2D g2 = (Graphics2D) g;
+		g2.setStroke(new BasicStroke(1));
+		g.setFont((extraLabelFont != null) ? extraLabelFont : getFont());
+		Object[] labels = GraphConstants
+				.getExtraLabels(view.getAllAttributes());
+		JGraph graph = (JGraph) this.graph.get();
+		if (labels != null)
+		{
+			for (int i = 0; i < labels.length; i++)
+				paintLabel(g, graph.convertValueToString(labels[i]),
+						getExtraLabelPosition(view, i),
+						false || !simpleExtraLabels);
+		}
+		if (graph.getEditingCell() != view.getCell())
+		{
+			g.setFont(getFont());
+			Object label = graph.convertValueToString(view);
+			if (label != null)
+			{
+				paintLabel(g, label.toString(), getLabelPosition(view), true);
+			}
+		}
+	}
+
+	/**
+	 * Paints the edge itself
+	 * @param g the graphics object being painted to
+	 */
+	protected void paintEdge(Graphics g)
+	{
+		g.setColor(getForeground());
+		if (lineWidth > 0) {
+			Graphics2D g2 = (Graphics2D) g;
+			int c = BasicStroke.CAP_BUTT;
+			int j = BasicStroke.JOIN_MITER;
+			g2.setStroke(new BasicStroke(lineWidth, c, j));
+			if (gradientColor != null && !preview) {
+				g2.setPaint(new GradientPaint(0, 0, getBackground(),
+						getWidth(), getHeight(), gradientColor, true));
+			}
+			if (view.beginShape != null) {
+				if (beginFill)
+					g2.fill(view.beginShape);
+				g2.draw(view.beginShape);
+			}
+			if (view.endShape != null) {
+				if (endFill)
+					g2.fill(view.endShape);
+				g2.draw(view.endShape);
+			}
+			if (lineDash != null) // Dash For Line Only
+				g2.setStroke(new BasicStroke(lineWidth, c, j, 10.0f,
+						lineDash, dashOffset));
+			if (view.lineShape != null)
+				g2.draw(view.lineShape);
+		}
+	}
+	
+	/**
+	 * Paints any selection effect applied to the edge
+	 * @param g the graphics object being painted to
+	 */
+	protected void paintSelection(Graphics g)
+	{
+		if (selected) { // Paint Selected
+			Graphics2D g2 = (Graphics2D) g;
+			g2.setStroke(GraphConstants.SELECTION_STROKE);
+			g2.setColor(highlightColor);
+			if (view.beginShape != null)
+				g2.draw(view.beginShape);
+			if (view.lineShape != null)
+				g2.draw(view.lineShape);
+			if (view.endShape != null)
+				g2.draw(view.endShape);
+		}
+	}
+
+	/**
+	 * Hook method for painting prior to default painting
+	 * @param g the graphics object being painted to
+	 */
+	protected void beforeEdgePaint(Graphics g) {
+		
+	}
+	
+	/**
+	 * Hook method for painting after the default painting
+	 * @param g the graphics object being painted to
+	 */
+	protected void afterEdgePaint(Graphics g) {
+		
+	}
+	
+
 	/**
 	 * Provided for subclassers to paint a selection border.
 	 */
