@@ -24,7 +24,11 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
 
+import javax.swing.BorderFactory;
+import javax.swing.JComponent;
+import javax.swing.JPanel;
 import javax.swing.SwingUtilities;
+import javax.swing.border.BevelBorder;
 
 import org.jgraph.JGraph;
 import org.jgraph.plaf.GraphUI;
@@ -68,6 +72,13 @@ public class EdgeView extends AbstractCellView {
 
 	protected transient Rectangle2D cachedBounds = null;
 
+	/** Whether or not pre 5.12.3.3 disconnectable behaviour is to be used.
+	 *  This allowed an edge to reconnect to another vertex ever when
+	 *  isDisconnectable was false for the edge. Set to false
+	 *  with isDisconnectable set to false for the edge forbids 
+	 *  any disconnection. Default is true. */
+	public static boolean LEGACY_DISCONNECTABLE = true;
+
 	/**
 	 * Constructs an empty edge view.
 	 */
@@ -888,6 +899,12 @@ public class EdgeView extends AbstractCellView {
 		 */
 		protected boolean edgeModified = false;
 
+		/**
+		 * Component that is used for highlighting cells if
+		 * the graph does not allow XOR painting.
+		 */
+		protected JComponent highlight = new JPanel();
+
 		public EdgeHandle(EdgeView edge, GraphContext ctx) {
 			this.graph = ctx.getGraph();
 			this.edge = edge;
@@ -905,6 +922,23 @@ public class EdgeView extends AbstractCellView {
 			reloadPoints(orig);
 			isEdgeConnectable = GraphConstants.isConnectable(edge
 					.getAllAttributes());
+			
+			// Configures the panel for highlighting ports
+			highlight = createHighlight();
+		}
+
+		/**
+		 * Creates the component that is used for highlighting cells if
+		 * the graph does not allow XOR painting.
+		 */
+		protected JComponent createHighlight()
+		{
+			JPanel panel = new JPanel();
+			panel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
+			panel.setVisible(false);
+			panel.setOpaque(false);
+
+			return panel;
 		}
 
 		protected void reloadPoints(EdgeView edge) {
@@ -960,6 +994,54 @@ public class EdgeView extends AbstractCellView {
 			}
 		}
 
+		/**
+		 * Highlights the given cell view or removes the highlight if
+		 * no cell view is specified.
+		 * 
+		 * @param graph
+		 * @param cellView
+		 */
+		protected void highlight(JGraph graph, CellView cellView)
+		{
+			if (cellView != null)
+			{
+				highlight.setBounds(getHighlightBounds(graph, cellView));
+
+				if (highlight.getParent() == null)
+				{
+					graph.add(highlight);
+					highlight.setVisible(true);
+				}
+			}
+			else
+			{
+				if (highlight.getParent() != null)
+				{
+					highlight.setVisible(false);
+					highlight.getParent().remove(highlight);
+				}
+			}
+		}
+
+		/**
+		 * Returns the bounds to be used to highlight the given cell view.
+		 * 
+		 * @param graph
+		 * @param cellView
+		 * @return
+		 */
+		protected Rectangle getHighlightBounds(JGraph graph, CellView cellView)
+		{
+			boolean offset = (GraphConstants.getOffset(cellView.getAllAttributes()) != null);
+			Rectangle2D r = (offset) ? cellView.getBounds() : cellView
+					.getParentView().getBounds();
+			r = graph.toScreen((Rectangle2D) r.clone());
+			int s = 3;
+
+			return new Rectangle((int) (r.getX() - s), (int) (r.getY() - s),
+					(int) (r.getWidth() + 2 * s), (int) (r.getHeight() + 2 * s));
+		}
+
 		public void overlay(Graphics g) {
 			if (edge != null && !firstOverlayCall && edge.isLeaf()) {
 				// g.setColor(graph.getBackground()); // JDK 1.3
@@ -972,11 +1054,24 @@ public class EdgeView extends AbstractCellView {
 				g2.scale(graph.getScale(), graph.getScale());
 				graph.getUI().paintCell(g, edge, edge.getBounds(), true);
 				g2.setTransform(oldTransform);
-				if (isSourceEditing() && edge.getSource() != null)
-					paintPort(g, edge.getSource());
-				else if (isTargetEditing() && edge.getTarget() != null)
-					paintPort(g, edge.getTarget());
+				
+				if (graph.isXorEnabled())
+				{
+					if (isSourceEditing() && edge.getSource() != null)
+						paintPort(g, edge.getSource());
+					else if (isTargetEditing() && edge.getTarget() != null)
+						paintPort(g, edge.getTarget());
+				}
 			}
+			
+			if (!graph.isXorEnabled())
+			{
+				if (isSourceEditing())
+					highlight(graph, edge.getSource());
+				else if (isTargetEditing())
+					highlight(graph, edge.getTarget());
+			}
+			
 			firstOverlayCall = false;
 		}
 
@@ -1127,15 +1222,38 @@ public class EdgeView extends AbstractCellView {
 			int y = event.getY();
 			// Detect hit on control point
 			int index = 0;
-			for (index = 0; index < r.length; index++) {
-				if (r[index].contains(x, y)) {
-					currentPoint = edge.getPoint(index);
-					currentIndex = index;
-					source = index == 0;
-					target = index == r.length - 1;
-					break;
+			for (index = 0; index < r.length; index++)
+			{
+				if (r[index].contains(x, y))
+				{
+					if (EdgeView.LEGACY_DISCONNECTABLE)
+					{
+						currentPoint = edge.getPoint(index);
+						currentIndex = index;
+						source = index == 0;
+						target = index == r.length - 1;
+						break;
+					}
+					else
+					{
+						if ((index > 0 && index < r.length - 1)
+								|| GraphConstants.isDisconnectable(edge
+										.getAllAttributes()))
+						{
+							currentPoint = edge.getPoint(index);
+							currentIndex = index;
+							source = index == 0;
+							target = index == r.length - 1;
+							break;
+						}
+						else
+						{
+							event.consume();
+						}
+					}
 				}
 			}
+			
 			// Detect hit on label
 			if (!isEditing() && graph.isMoveable()
 					&& GraphConstants.isMoveable(edge.getAllAttributes())
@@ -1207,6 +1325,7 @@ public class EdgeView extends AbstractCellView {
 		}
 
 		public void mouseDragged(MouseEvent event) {
+			Rectangle2D dirty = edge.getBounds();
 			Point2D p = graph.fromScreen(new Point(event.getPoint()));
 			// Move Label
 			if (label || currentLabel >= 0) {
@@ -1250,7 +1369,6 @@ public class EdgeView extends AbstractCellView {
 							p = new Point2D.Double(x - p0.getX(), y - p0.getY());
 						}
 					}
-					Rectangle2D dirty = edge.getBounds();
 					if (label) {
 						edge.setLabelPosition(p);
 					} else {
@@ -1260,9 +1378,9 @@ public class EdgeView extends AbstractCellView {
 					if (graph.isXorEnabled()) {
 						overlay(graph.getGraphics());
 					} else {
-						graph.repaint((int) dirty.getX(), (int) dirty
-								.getY(), (int) dirty.getWidth(),
-								(int) dirty.getHeight());
+						graph.repaint((int) dirty.getX() - 1,
+								(int) dirty.getY() - 1, (int) dirty.getWidth() + 2,
+								(int) dirty.getHeight() + 2);
 					}
 				}
 			} else if (isEditing() && currentPoint != null) {
@@ -1306,7 +1424,6 @@ public class EdgeView extends AbstractCellView {
 									null) || graph.isPreviewInvalidNullPorts());
 					if (acceptSource || acceptTarget || !(source || target)) {
 						edgeModified = true;
-						Rectangle2D dirty = edge.getBounds();
 						if (edge.getSource() != null) {
 							dirty.add(edge.getSource().getParentView()
 									.getBounds());
@@ -1358,12 +1475,21 @@ public class EdgeView extends AbstractCellView {
 								dirty.add(edge.getTarget().getParentView()
 										.getBounds());
 							}
+							dirty = graph.toScreen((Rectangle2D) dirty.clone());
 							graph.repaint((int) dirty.getX(), (int) dirty
 									.getY(), (int) dirty.getWidth(),
 									(int) dirty.getHeight());
 						}
 					}
 				}
+				else if (!graph.isXorEnabled())
+				{
+					dirty.add(edge.getBounds());
+					dirty = graph.toScreen((Rectangle2D) dirty.clone());
+					graph.repaint((int) dirty.getX(), (int) dirty
+							.getY(), (int) dirty.getWidth(),
+							(int) dirty.getHeight());
+				}
 			}
 		}
 		
@@ -1486,6 +1612,7 @@ public class EdgeView extends AbstractCellView {
 
 		// Handle mouse released event
 		public void mouseReleased(MouseEvent e) {
+			highlight(graph, null); // removes the highlight
 			boolean clone = e.isControlDown() && graph.isCloneable();
 			GraphModel model = graph.getModel();
 			Object source = (edge.getSource() != null) ? edge.getSource()
